/**
 * \addtogroup 24XX32
 * @{
 */

/*****************************************************************************/
/**
 * \file 24xx32.c
 * 24xx32 serial eeprom routines for 8051 and Keil C51.
 * \author Copyright (c) 2010, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.0
 * \date 30-06-10
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "24xx32.h"


/*****************************************************************************
 *
 *                         Public Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                               _24xx32_init()
 *
 *****************************************************************************/
void _24xx32_init(void)
{
    /* 
     * Send software reset sequence so as to I2C-protocol-reset all slaves.
     */
    i2c_transmit_reset();

    return;
}


/*****************************************************************************
 *
 *                            _24xx32_read_byte()
 *
 *****************************************************************************/
unsigned char _24xx32_read_byte(unsigned char device,
                                unsigned int address)
{
    unsigned char byte;

    /* 
     * Read byte from serial eeprom device at specified address.
     */

    /* Transmit I2C start condition. */
    i2c_transmit_start();

    /* Select I2C eeprom storage device for word pointer write. */
    i2c_transmit_byte(_24XX32_EEPROM_ID | (device << 1) | _24XX32_WRITE);

    /* Write into word pointer to address required memory element */
    i2c_transmit_byte((unsigned char)(address >> 8));

    /* Write into word pointer to address required memory element */
    i2c_transmit_byte((unsigned char)(address & 0xFF));

    /* Transmit I2C restart condition. */
    i2c_transmit_restart();

    /* Select required I2C eeprom storage device for read. */
    i2c_transmit_byte(_24XX32_EEPROM_ID | (device << 1) | _24XX32_READ);

    /* 
     * Retrieve single byte of stored data. (I2C_NAK indicates final
     * retrieval of a, possibly extended, reception sequence).
     */
    byte = i2c_receive_byte(I2C_NAK);

    /* Transmit I2C stop condition. */
    i2c_transmit_stop();

    return byte;
}


/*****************************************************************************
 *
 *                            _24xx32_read_next_byte()
 *
 *****************************************************************************/
unsigned char _24xx32_read_next_byte(unsigned char device)
{
    unsigned char byte;

    /* 
     * Read byte from serial eeprom device at current read address.
     */

    /* Transmit I2C start condition. */
    i2c_transmit_start();

    /* Select required eeprom storage device for read. */
    i2c_transmit_byte(_24XX32_EEPROM_ID | (device << 1) | _24XX32_READ);

    /* 
     * Retrieve single byte of stored data. (I2C_NAK indicates final
     * retrieval of a, possibly extended, reception sequence).
     */
    byte = i2c_receive_byte(I2C_NAK);

    /* Transmit I2C stop condition. */
    i2c_transmit_stop();

    return byte;
}


/*****************************************************************************
 *
 *                            _24xx32_read_page()
 *
 *****************************************************************************/
void _24xx32_read_page(unsigned char device,
                       unsigned char page,
                       unsigned char num_bytes, 
                       unsigned char * buffer)
{
    unsigned char i = 0;
    unsigned int address = page * _24XX32_WORDS_PER_PAGE;

    /* 
     * Read buffer from serial eeprom device from specified page.
     */

    /* Transmit I2C start condition. */
    i2c_transmit_start();

    /* Select I2C eeprom storage device for word pointer write. */
    i2c_transmit_byte(_24XX32_EEPROM_ID | (device << 1) | _24XX32_WRITE);

    /* Write into word pointer to address required memory element */
    i2c_transmit_byte((unsigned char)(address >> 8));

    /* Write into word pointer to address required memory element */
    i2c_transmit_byte((unsigned char)(address & 0xFF));

    /* Transmit I2C restart condition. */
    i2c_transmit_restart();

    /* Select required I2C eeprom storage device for read. */
    i2c_transmit_byte(_24XX32_EEPROM_ID | (device << 1) | _24XX32_READ);

    /* Retrive all but last byte of page from serial eeprom device. */
    for (i = 0; i < num_bytes - 1; i++)
    {
        buffer[i] = i2c_receive_byte(I2C_ACK);
    }

    /* 
     * Retrieve single byte of stored data. (I2C_NAK indicates final
     * retrieval of a, possibly extended, reception sequence).
     */
    buffer[num_bytes - 1] = i2c_receive_byte(I2C_NAK);

    /* Transmit I2C stop condition. */
    i2c_transmit_stop();

    return;
}


/*****************************************************************************
 *
 *                           _24xx32_write_byte()
 *
 *****************************************************************************/
void _24xx32_write_byte(unsigned char device,
                        unsigned int address,
                        unsigned char byte)
{
    bit reply = I2C_NAK;

    /* 
     * Write byte to serial eeprom device at specified address.
     */

    /* Transmit I2C start condition. */
    i2c_transmit_start();

    /* Select required I2C eeprom storage device for write. */
    i2c_transmit_byte(_24XX32_EEPROM_ID | (device << 1) | _24XX32_WRITE);
  
    /* Write into word pointer to address required memory element */
    i2c_transmit_byte((unsigned char)(address >> 8));

    /* Write into word pointer to address required memory element */
    i2c_transmit_byte((unsigned char)(address & 0xFF));

    /* Transmit data byte to be written to serial eeprom device. */
    i2c_transmit_byte(byte);

    /* Transmit I2C stop condition. */
    i2c_transmit_stop();


    /*
     * Continuously poll EEPROM device until acknowledgement that 
     * previously scheduled internal write operation has completed.
     * (Scheduled writes can typically require up to 5ms to complete.) 
     */
    do
    {
        /*
         * Continuously transmit I2C start condition, then transmit control
         * word for a device write, testing whether EEPROM responds with an
         * internal write completion acknowledgement, or not. Subsequently, 
         * transmit I2C stop condition and then, based on ack test results, 
         * either poll again, or exit.
         */

        /* Transmit I2C start condition. */
        i2c_transmit_start();

        /* Test for EEPROM write completion. */
        reply = i2c_transmit_byte_then_test_for_ack(_24XX32_EEPROM_ID |
                                                    (device << 1) | 
                                                    _24XX32_WRITE);

        /* Transmit I2C stop condition. */
        i2c_transmit_stop();

    } while (reply == I2C_NAK);

    return;
}


/*****************************************************************************
 *
 *                           _24xx32_write_page()
 *
 *****************************************************************************/
void _24xx32_write_page(unsigned char device,
                        unsigned char page,
                        unsigned char num_bytes,
                        unsigned char * buffer)
{
    unsigned char i = 0;
    bit reply = I2C_NAK;
    unsigned int address = page * _24XX32_WORDS_PER_PAGE;

    /* 
     * Write buffer to serial eeprom device at specified page.
     */

    /* Transmit I2C start condition. */
    i2c_transmit_start();

    /* Select required I2C eeprom storage device for write. */
    i2c_transmit_byte(_24XX32_EEPROM_ID | (device << 1) | _24XX32_WRITE);
  
    /* Write into word pointer to address required memory element */
    i2c_transmit_byte((unsigned char)(address >> 8));

    /* Write into word pointer to address required memory element */
    i2c_transmit_byte((unsigned char)(address & 0xFF));

    /* Transmit buffer bytes to be written to serial eeprom device. */
    for (i = 0; i < num_bytes; i++)
    {
        i2c_transmit_byte(buffer[i]);
    }

    /* Transmit I2C stop condition. */
    i2c_transmit_stop();


    /*
     * Continuously poll EEPROM device until acknowledgement that 
     * previously scheduled internal write operation has completed.
     * (Scheduled writes can typically require up to 5ms to complete.) 
     */
    do
    {
        /*
         * Continuously transmit I2C start condition, then transmit control
         * word for a device write, testing whether EEPROM responds with an
         * internal write completion acknowledgement, or not. Subsequently, 
         * transmit I2C stop condition and then, based on ack test results, 
         * either poll again, or exit.
         */

        /* Transmit I2C start condition. */
        i2c_transmit_start();

        /* Test for EEPROM write completion. */
        reply = i2c_transmit_byte_then_test_for_ack(_24XX32_EEPROM_ID |
                                                    (device << 1) | 
                                                    _24XX32_WRITE);

        /* Transmit I2C stop condition. */
        i2c_transmit_stop();

    } while (reply == I2C_NAK);

    return;
}


/** @} */
